@@ -1,13 +1,16 @@
+import aiofiles
 import copy
 import fnmatch
+import msgpack
 import logging
+import pathlib
 import re
 import yaml
 
 from . import DEFAULT_TARGET_DELIM
 from . import args
 from collections.abc import Mapping, MutableMapping, Sequence
-from typing import Any, Dict, Iterable, List, Set
+from typing import Any, Dict, Iterable, List, Set, Iterator
 
 log = logging.getLogger(__name__)
 
@@ -141,18 +144,19 @@ class NamespaceDict(dict):
         NamespaceDict(**kwargs) -> new namespaced dictionary initialized with the name=value pairs
             in the keyword argument list.  For example:  NamespaceDict(one=1, two=2)
         """
-        if seq is None:
-            super().__init__(**kwargs)
-        else:
-            super().__init__(seq, **kwargs)
+        if seq is not None:
+            for k in seq:
+                self[k] = seq[k]
+        for k, v in kwargs.items():
+            self[k] = v
 
     def __setattr__(self, k: str, v: Any):
-        if isinstance(v, dict) and not isinstance(v, NamespaceDict):
+        if isinstance(v, Iterable) and not isinstance(v, NamespaceDict):
             v = NamespaceDict(v)
         self[k] = v
 
     def __getattr__(self, k: str):
-        if k.startswith("__") and k.endswith("__"):
+        if k.startswith("_"):
             return super().__getattribute__(k)
         return self[k]
 
@@ -165,6 +169,71 @@ class NamespaceDict(dict):
         return NamespaceDict(copy.deepcopy(self.copy(), memodict))
 
 
+class FileDict(Mapping):
+    """
+    A dictionary type that stores it's information in a file
+    """
+
+    def __init__(self, cache: str, seq: Iterable = None, **kwargs):
+        self.cache = pathlib.Path(cache)
+        if seq:
+            for k in seq:
+                self[k] = seq[k]
+        for k, v in kwargs.items():
+            self[k] = v
+
+    def get(self, k, default=None):
+        if self.cache.exists():
+            with open(self.cache, "rb+") as fh_:
+                return msgpack.loads(fh_.read()).get(k, default)
+        else:
+            return default
+
+    async def aget(self, k, default=None):
+        if self.cache.exists():
+            async with aiofiles.open(self.cache, "rb+") as fh_:
+                return msgpack.loads(await fh_.read()).get(k, default)
+        else:
+            return default
+
+    async def aset(self, k, v):
+        if self.cache.exists():
+            async with aiofiles.open(self.cache, "rb+") as fh_:
+                data = msgpack.loads(await fh_.read())
+        else:
+            data = {}
+        data[k] = v
+        async with aiofiles.open(self.cache, "wb+") as fh_:
+            await fh_.write(msgpack.dumps(data))
+
+    def __setitem__(self, k, v):
+        if self.cache.exists():
+            with open(self.cache, "rb+") as fh_:
+                data = msgpack.loads(fh_.read())[k]
+        else:
+            data = {}
+        data[k] = v
+        with open(self.cache, "wb+") as fh_:
+            fh_.write(msgpack.dumps(data))
+
+    def __getitem__(self, k):
+        with open(self.cache, "rb+") as fh_:
+            return msgpack.loads(fh_)[k]
+
+    async def __aiter__(self):
+        async with aiofiles.open(self.cache, "rb+") as fh_:
+            for k in msgpack.loads(await fh_.read()):
+                yield k
+
+    def __iter__(self) -> Iterator:
+        with open(self.cache, "rb+") as fh_:
+            return iter(msgpack.loads(fh_))
+
+    def __len__(self) -> int:
+        with open(self.cache, "rb+") as fh_:
+            return len(msgpack.loads(fh_.read()))
+
+
 def __change_case(data, attr, preserve_dict_class=False):
     """
     Calls data.attr() if data has an attribute/method called attr.
@@ -542,13 +611,22 @@ def subdict_match(
         if not ret and wildcard:
             for key in target:
                 if isinstance(target[key], dict):
-                    if _dict_match(target[key], pattern,):
+                    if _dict_match(
+                        target[key],
+                        pattern,
+                    ):
                         return True
                 elif isinstance(target[key], list):
                     for item in target[key]:
-                        if _match(item, pattern,):
+                        if _match(
+                            item,
+                            pattern,
+                        ):
                             return True
-                elif _match(target[key], pattern,):
+                elif _match(
+                    target[key],
+                    pattern,
+                ):
                     return True
         return ret
 
@@ -586,7 +664,10 @@ def subdict_match(
             # We are matching a single component to a single list member
             for member in match:
                 if isinstance(member, dict):
-                    if _dict_match(member, matchstr,):
+                    if _dict_match(
+                        member,
+                        matchstr,
+                    ):
                         return True
                 if _match(member, matchstr):
                     return True
